/**
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) 2016-2020 OX Software GmbH
 */
define('oxguard/files/register', [
    'io.ox/core/extensions',
    'io.ox/core/capabilities',
    'oxguard/oxguard_core',
    'io.ox/core/folder/api',
    'oxguard/files/util',
    'io.ox/core/dropzone',
    'oxguard/auth',
    'gettext!oxguard',
    'io.ox/core/extPatterns/links',
    'less!oxguard/style'
], function (ext, capabilities, oxguard, folderAPI, util, dropzone, auth_core, gt, links) {

    'use strict';

    // For mobile, add a decrypt button
    ext.point('io.ox/files/details').extend({
        index: 500,
        id: 'downloadGuardsmall',
        draw: function (baton) {
            if (_.device('small')) {
                if (_.device('ios')) return;   // ios doesn't allow downloading.
                if (baton.data.ogfile === true) {
                    var download = $('<a href="#" class="btn btn-default">' + gt('Download Decrypted') + '</a>');
                    download.click(function (ev) {
                        ev.preventDefault();
                        auth_core.authorize().then(function (data) {
                            require (['oxguard/files/downloader'], function (downloader) {
                                metrics('guard', 'decrypt & download');
                                downloader.viewFile(baton, 'download', data);
                            });
                        });
                    });
                    $(this).append(download);
                }
            }
        }
    });

    ///////////  Main toolbar in drive

    //////   Links
    // Add link for add and encrypt local file
    new links.ActionLink('io.ox/files/links/toolbar/default', {
        index: 110,
        id: 'uploadEncrypt',
        label: gt('Add and encrypt local file'),
        ref: 'io.ox/files/actions/uploadEncrypt'
    });

    ext.point('io.ox/files/classic-toolbar/links').extend(new links.Link({
        id: 'remencrypt',
        index: 550,
        label: gt('Remove encryption'),
        ref: 'oxguard/remencrypt',
        mobile: 'lo',
        section: 'guard'
    }));

    ext.point('io.ox/files/classic-toolbar/links').extend(new links.Link({
        id: 'encrypt',
        index: 550,
        label: gt('Encrypt'),
        ref: 'oxguard/encrypt',
        mobile: 'lo',
        section: 'guard'
    }));

    // Change send copy to handle Guard emails
    ext.point('io.ox/files/actions/send').extend({
        id: 'og_stop_send',
        index: 1,
        requires: function (e) {
            if (e.collection.has('some', 'items') && !_.isEmpty(e.baton.data)) {
                var toCheck = e.baton.models === null ? e.baton.model : e.baton.models;
                if (util.hasEncrypted(toCheck)) {
                    e.stopPropagation();
                    return false;
                }
            }
        }
    });

    ext.point('io.ox/files/classic-toolbar/links').extend(new links.Link({
        id: 'sendcopy',
        index: 1200,
        prio: 'lo',
        mobile: 'lo',
        label: gt('Send by mail'),
        ref: 'oxguard/sendcopy',
        section: 'share'
    }));

    //////////////  Viewer
    /// Links

    ext.point('io.ox/core/viewer/actions/toolbar/popoutstandalone').extend({
        id: 'popoutstandalone_guardcheck',
        index: 1,
        requires: function (e) {
            if (e.collection.has('one')) {
                if (e.baton.model.isEncrypted()) {
                    e.stopPropagation();
                    return false;
                }
            }
        }
    });

    ext.point('io.ox/core/viewer/toolbar/links/drive').extend(new links.Link({
        id: 'GuardViewerEncrypt',
        index: 200,
        label: gt('Encrypt'),
        ref: 'oxguard/encrypt',
        mobile: 'lo'
    }));

    ext.point('io.ox/core/viewer/toolbar/links/guardDrive').extend(new links.Link({
        id: 'GuardViewerRemEncrypt',
        index: 200,
        label: gt('Remove encryption'),
        ref: 'oxguard/remencrypt',
        mobile: 'lo'
    }));

    ext.point('io.ox/core/viewer/toolbar/links/guardDrive').extend(new links.Link({
        id: 'GuardViewerDownload',
        index: 201,
        label: gt('Download Decrypted'),
        ref: 'oxguard/download',
        section: 'export',
        mobile: 'lo'
    }));

    ext.point('io.ox/core/viewer/toolbar/links/guardDrive').extend(new links.Link({
        id: 'GuardViewerDownloadEncr',
        index: 202,
        label: gt('Download Encrypted'),
        ref: 'oxguard/downloadEncrypted',
        section: 'export',
        mobile: 'lo'
    }));

    ext.point('io.ox/core/viewer/toolbar/links/guardDrive').extend(new links.Link({
        prio: 'hi',
        mobile: 'lo',
        label: gt('Pop out'),
        icon: 'fa  fa-external-link-square',
        ref: 'oxguard/popoutstandalone',
        customize: function () {
            this.addClass('viewer-toolbar-popoutstandalone')
                .attr({
                    tabindex: '1',
                    title: gt('Pop out standalone viewer'),
                    'aria-label': gt('Pop out standalone viewer')
                });
        }
    }));

    // Versions
    // View is removed from versions currently
    /*ext.point('io.ox/files/versions/links/inline').extend(new links.Link({
        id: 'GuardView',
        index: 90,
        label: gt('View'),
        ref: 'oxguard/versionView'
    }));
    */

    ext.point('io.ox/files/versions/links/inline').extend(new links.Link({
        id: 'GuardOpen',
        index: 100,
        label: gt('Decrypt and Open'),
        ref: 'oxguard/open'
    }));

    ext.point('io.ox/files/versions/links/inline').extend(new links.Link({
        id: 'GuardDownload',
        index: 200,
        label: gt('Download Decrypted'),
        ref: 'oxguard/download'
    }));

    // Remove the loc portion of the name in version list
    ext.point('io.ox/files/details/version').extend({
        index: 1,
        id: 'GuardVersion',
        draw: function () {}
    });

    //// Actions

    // Upload new encrypted file
    new links.Action('io.ox/files/actions/uploadEncrypt', {
        requires: function (e) {
            if (util.isMailvelope ()) return false;
            return e.baton.app.folder.getData().then(function (data) {
                //hide for virtual folders (other files root, public files root)
                var virtual = _.contains(['14', '15'], data.id);
                //no new files in trash folders
                return folderAPI.can('create', data) && !virtual && !folderAPI.is('trash', data);
            });
        },
        action: function (baton) {
            ensureSetup().then(function () {
                baton.file_options = { 'params': { 'cryptoAction': 'Encrypt' }};
                ext.point('io.ox/files/actions/upload').invoke('action', this, baton);
            });
        }
    });

    new links.Action('oxguard/downloadEncrypted', {
        requires: function (e) {
            if (util.isMailvelope ()) return false;
            if (e.baton.data) {
                if (isOGFile(e.baton.data)) {
                    return (true);
                }
            }
            return false;
        },
        multiple: function (list) {
            list.map(function (file) {
                // For encrypted, wipe the cryptoAuth and cryptoAction
                if (file && file.params) {
                    file.params = {};
                }
            });
            ext.point('io.ox/files/actions/download').invoke('multiple', this, list);

        }
    });

    ext.point('io.ox/files/actions/open').replace({
        id: 'default',
        requires: function (e) {
            if (e.collection.has('multiple')) return false;
            if (e.baton.data) {
                if (isOGFile(e.baton.data)) {
                    return (false);
                }
            }
            // 'description only' items
            return !_.isEmpty(e.baton.data.filename) || e.baton.data.file_size > 0;
        }
    });

    // Replacement popout for Guard
    new links.Action('oxguard/popoutstandalone', {
        requires: function (e) {
            if (e.collection.has('one')) {
                var currentApp = ox.ui.App.getCurrentApp().getName();
                // detail is the target of popoutstandalone, no support for mail attachments
                return (currentApp !== 'io.ox/files/detail' && e.baton.model.isEncrypted());
            }
            return false;
        },
        action: function (e) {
            ox.launch('io.ox/files/detail/main').done(function () {
                var node = this.getWindowNode();
                this.setTitle(e.data.filename);
                var app = this;
                ox.load(['io.ox/mail/actions/viewer']).done(function (action) {
                    action({ files: [e.data], app: app, container: node, standalone: true });
                });
            });
        }
    });

    // View link within versions
    new links.Action('oxguard/versionView', {
        requires: function (e) {
            return (isOGFile(e.baton.data));
        },
        multiple: function (list) {
            debugger;
            viewFile (list);
        }
    });

    new links.Action('oxguard/sendcopy', {
        requires: function (e) {
            if (util.isMailvelope ()) return false;
            //all files must be encrypted
            var toCheck = e.baton.models === null ? e.baton.model : e.baton.models;
            return e.collection.has('some') && util.hasEncrypted(toCheck) && util.isNotAttachment(toCheck) && capabilities.has('webmail');
        },
        multiple: function (list) {
            list.map(function (file) {
                if (file.source === 'guardDrive') window.oxguarddata.action = 'fileAttach';
            });
            ext.point('io.ox/files/actions/send').invoke('multiple', this, list);
        }
    });

    new links.Action('oxguard/remencrypt', {
        requires: function (e) {
            //all files must be encrypted
            var toCheck = e.baton.models === null ? e.baton.model : e.baton.models;
            return e.collection.has('some') && util.encryptedOnly(toCheck) && util.isNotAttachment(toCheck);
        },
        multiple: function (list, baton) {
            ensureSetup().then(function () {
                auth_core.authorize().then(function (auth) {
                    for (var file in list) {
                        list[file].file_options = {
                            params: {
                                cryptoAction: 'Decrypt',
                                cryptoAuth: auth
                            }
                        };
                    }
                    require(['io.ox/files/api'], function (fileApi) {
                        fileApi.copy(list, list[0].folder_id, false)
                        .then(function (e) {
                            if (e.length == list.length) {
                                for (var i in list) {
                                    if (e[i].data) {
                                        fileApi.remove([list[i]], true);
                                    }
                                }
                            } else {
                                if (_.isString(e)) {
                                    require (['io.ox/core/notifications'], function (notify) {
                                        notify.yell('error', e);
                                    });
                                }
                            }
                        });
                    });
                });
                baton.openedBy = 'io.ox/files';
            });
        }
    });

    new links.Action('oxguard/open', {
        requires: function (e) {
            if (e.baton.data) {
                if (isOGFile(e.baton.data)) {
                    return (true);
                }
            }
            return (false);
        },
        multiple: function (list) {
            auth_core.authorize().then(function (auth) {
                var params = {
                    'cryptoAction': 'Decrypt',
                    'cryptoAuth': auth
                };
                require (['io.ox/files/api'], function (fileApi) {
                    _(list).each(function (file) {
                        window.open(fileApi.getUrl(file, 'open', { params: params }));
                    });
                });
                metrics('guard', 'decrypt & open');
            });
        }
    });

    new links.Action('oxguard/download', {
        requires: function (e) {
            if (_.device('ios')) return false;
            var toCheck = e.baton.models === null ? e.baton.model : e.baton.models;
            if (toCheck === undefined) return false;
            return e.collection.has('some') && util.encryptedOnly(toCheck) && util.isNotAttachment(toCheck);
        },
        multiple: function (list) {
            auth_core.authorize().then(function (auth) {
                for (var file in list) {
                    list[file].filename = (list[file].filename ? list[file].filename.replace('.pgp', '') : '');
                    list[file].params = {
                        'cryptoAction': 'Decrypt',
                        'cryptoAuth': auth
                    };
                    list[file].cache = false;
                }
                ext.point('io.ox/files/actions/download').invoke('multiple', this, list);
                metrics('guard', 'decrypt & download');
            });
        }
    });

    ////// Main listview handling

    // Call for viewer.  Check if Guard file
    ext.point('io.ox/files/actions/viewer').extend({
        id: 'guardview',
        index: 1,
        requires: function (e) {
            var toCheck = e.baton.models === null ? e.baton.model : e.baton.models;
            if (toCheck === undefined) return false;
            return e.collection.has('some') && util.encryptedOnly(toCheck);
        },
        action: function (baton) {
            if (isOGFile(baton.data)) {
                viewFile(baton);
                baton.preventDefault();
            }
        }
    });

    // Remove file decode parameters from file models once viewer closes
    ext.point('io.ox/core/viewer/toolbar/links/guardDrive').extend({
        id: 'guardCleanup',
        index: 1,
        draw: function (baton) {
            baton.context.viewerEvents.listenTo(baton.context.viewerEvents, 'viewer:beforeclose', function () {
                _.map(baton.context.collection.models, function (file) {
                    file.set('file_options', {});
                });
            });
            // Blank insert for 2.6   Plan to add icon in next release
            $(this).append('<a href="#" style="padding: 0 0 0 1px; min-width: 1px;"></a>');
            //$(this).append('<a href="#" style="padding: 0 0 0 16px; min-width: 1px;"><i class="fa fa-lock" aria-hidden="true"></i></a>');
            $(this).addClass('lockLeft');
        }
    });

    // Handle viewing Guard file
    function viewFile (baton) {
        if (isOGFile (baton.data)) {
            auth_core.authorize().then(function (auth) {
                require (['oxguard/files/downloader'], function (downloader) {
                    downloader.viewFile(baton, 'view', auth);
                });
            });
        }
    }

    // Add extra drop zone for upload new Guard file
    ext.point('io.ox/files/mediator').replace({
        id: 'files-dropzone',
        index: 1000000000000,
        setup: function (app) {
            // desktop only
            if (!_.device('desktop')) return;

            var zone = new dropzone.Inplace({
                caption: gt('Drop files here to upload')
            });

            var zone2 = new dropzone.Inplace({
                caption: gt('Drop files here to encrypt')
            });

            zone.on({
                'drop': function (files) {
                    require(['io.ox/files/upload/main'], function (fileUpload) {
                        fileUpload.setWindowNode(app.getWindowNode());
                        fileUpload.create.offer(files, { folder: app.folder.get() });
                    });
                }
            });

            zone2.on({
                'drop': function (files) {
                    ensureSetup().then(function () {
                        metrics('guard', 'dropfile upload & encrypt');
                        require(['io.ox/files/upload/main'], function (fileUpload) {
                            fileUpload.setWindowNode(app.getWindowNode());
                            fileUpload.create.offer(files, { folder: app.folder.get(), 'params': { 'cryptoAction': 'Encrypt' }});
                        });
                    });
                }
            });

            if (util.isMailvelope ()) {
                app.getWindowNode().find('.list-view-control').append(
                        (zone.render().$el).addClass('abs'));
            } else {
                app.getWindowNode().find('.list-view-control').append(
                        (zone.render().$el).addClass('abs').css('height', '50%')
                    ).append(
                        (zone2.render().$el).addClass('abs').css('top', '50%')
                    );
            }

        }
    });

    ////// Functions / util

    function ensureSetup() {

        if (!util.isGuardConfigured()) {
            oxguard.metrics('drive', 'create-keys');
            return oxguard.createKeysWizard(true);
        }

        return $.when();
    }

    ////////  Encrypting files

    new links.Action('oxguard/encrypt', {
        requires: function (e) {
            if (util.isMailvelope ()) return false;
            //all files must be unencrypted
            if (e.collection.has('folders')) return false;
            return e.collection.has('some') && !util.encryptedOnly(e.baton.models);
        },
        multiple: function (list) {
            ensureSetup().then(function () {
                for (var file in list) {
                    list[file].file_options = {
                        params: {
                            cryptoAction: 'Encrypt'
                        }
                    };
                }
                require(['io.ox/files/api'], function (fileApi) {
                    fileApi.copy(list, list[0].folder_id, false)
                    .then(function (e) {
                        if (e.length == list.length) {
                            for (var i in list) {
                                if (e[i].data) {
                                    fileApi.remove([list[i]], true);
                                }
                            }
                        } else {
                            if (_.isString(e)) {
                                require (['io.ox/core/notifications'], function (notify) {
                                    notify.yell('error', e);
                                });
                            }
                        }
                    });
                });
            });

        }
    });

    /////////////////  Util, prompts, etc

    function metrics (target, action) {
        require(['io.ox/metrics/main'], function (metrics) {
            if (!metrics.isEnabled()) return;
            metrics.trackEvent({
                app: 'drive',
                target: target,
                type: 'click',
                action: action
            });
        });
    }

    /*
    function reload () {
        require(['io.ox/files/main'], function (main) {
            main.getApp().listView.reload();
        });
    }

    */
    function isOGFile(file) {
        try {
            if (file.filename === null) return (false);
            if (file.meta !== undefined) {
                if (file.meta.Encrypted) return (true);
            }
            if ((file.filename.indexOf('.grd') > 0) || (file.filename.indexOf('.grd2') > 0) || (file.filename.indexOf('.pgp') > 0)) return (true);
        } catch (e) {
            return (false);
        }
        return (false);
    }

    return {
        viewFile: viewFile
    };
});
