/* All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) 2016-2020 OX Software GmbH
 */
define('oxguard/pgp/checker', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/links',
    'oxguard/core/og_http',
    'settings!oxguard',
    'gettext!oxguard'
], function (ext, links, http, settings, gt) {
    'use strict';

    var firstload = true;

    // Do the check if the email is PGP email.  Check for inline, MIME, or signature
    function checkPGP (baton, location) {
        if (settings.get('cryptoProvider') === 'mailvelope') return;

        if (baton.data.PGPDecoded === true) {
            require (['oxguard/pgp/reader'], function (reader) {
                reader.addIcons(baton);
            });
            if (window.oxguarddata.settings.oxguard === false) { // If not guard user, display upsell if any
                var loc = baton.view.$el;
                if (loc.find('.guardUpsell').length < 1) { // Make sure not already drawn
                    require (['oxguard/mail/upsell'], function (upseller) {
                        upseller.upsell(loc);
                    });
                }
            }
            return; // If already decoded
        }
        if (baton.view.model === null) {
            return;
        }
        var headers = baton.view.model.get('headers');
        if (headers === undefined) {// not yet loaded
            return;
        }
        // Init
        if (baton.data.headers === undefined) { // If email wasn't fully loaded into baton, do so now.
            baton.data = baton.view.model.toJSON();
        }
        var mail = baton.data;
        if (mail.results !== undefined) return;
        var attachments = baton.data.attachments;
        if (attachments === undefined) return;
        if (headers['X-OXGUARD-GUEST'] !== undefined) {
            baton.view.model.set('pgp_found', true);
            pgpAttach(baton);
            pgpFound.call(this, baton);
        }
        for (var i = 0; i < attachments.length; i++) {
            //var top = (attachments[i].size < 30) ? attachments[i].size : 30;
            if (attachments[i].content_type.toLowerCase().indexOf('pgp-encrypted') > -1) {
                baton.view.model.set('pgp_found', true);
                pgpAttach(baton);
                pgpFound.call(this, baton, location);
            }
            if (attachments[i].display = 'inline') {
                if (attachments[i].content !== null) {
                    if (attachments[i].content.indexOf('-----BEGIN PGP MESSAGE-----') > -1) {
                        baton.view.model.set('pgp_found', true);
                        inlineFound.call(this, baton, i, location);
                    }
                    if (attachments[i].content.indexOf('-----BEGIN PGP SIGNED') > -1) {
                        verify(i, baton, true, location);
                    }
                }

            }
            if (attachments[i].content_type.toLowerCase().indexOf('pgp-signature') > -1) {
                verify(i, baton, false, location);
                attachments[i].disp = 'none';  // Don't display the signature attachment
            }
        }
    }

    // Verify signature
    function verify (index, baton, isinline, location) {
        if (baton.signatureDone) return;
        var params = '&folder=' + baton.data.folder_id +
        '&userid=' + ox.user_id + '&cid=' + ox.context_id +
        '&emailid=' + baton.data.id + '&attach=' + index;
        if (isinline === true) params += '&inline=true';
        http.get(ox.apiRoot + '/oxguard/pgpmail?action=verify', params)
        .done(function (data) {
            baton.signatureDone = true;
            var results = JSON.parse(data);
            var result = results.result ? JSON.parse(results.result) : results;
            var signedicon = '';
            if (result.verified === true) {
                signedicon = $('<i class="oxguard_icon_fa fa-pencil-square-o fa guard_signed" aria-hidden="true" title="' + gt('This Email was signed and verified.') + '"/>');
                if (isinline) {
                    var newbody = $('<section class="body user-select-text">').append(result.content);
                    $(location).replaceWith(newbody);
                }
            } else if (result.error !== null && result.missingPublicKey) {
                signedicon = $('<i class="oxguard_icon_fa fa-pencil-square-o fa guard_signed" style="color:lightgrey" aria-hidden="true" title="' + gt('This Email was signed but unable to verify.') + '"/>');
            } else if (!result.verified && result.signature) {
                signedicon = $('<i class="oxguard_icon_fa_alert fa-pencil-square fa guard_signed" aria-hidden="true" title="' + gt('This Email was signed and failed verification.') + '"/>');
            } else {
                return;
            }
            if (_.device('small')) {
                signedicon.css('margin-right', '35px');
            } else {
                signedicon.css('margin-right', '-9px');
            }
            //FIXME: when switching mail-detail very fast, the view might already be disposed
            if (baton.view.$el === null) return;
            var detail = baton.view.$el.find('.detail-view-header');
            if (detail.find('.guard_signed').length < 1) {  // add icon if not already added
                var date = detail.find('.date');
                date.after(signedicon);
            }
            if (baton.data.attachments.length === 2) {
                detail.find('.fa-paperclip').hide();
                baton.data.attachment = false;  // Hide paperclip if just signature
            }
        });
    }

    function busy (location, id) {
        var working = $('<br/><div class="oxguard_error" id = "error' + id + '"><i class="fa fa-spinner fa-spin" id="busygif" style="padding:20px;"></i></div>');
        $(location).replaceWith(working);
    }

    // Inline is found, send for decode
    function inlineFound (baton, index, location) {

        var goFunction = function (id) {
            $('#error' + id).show();
            var password = ($('#oxgrpass' + id).val() === undefined) ? '' : $('#oxgrpass' + id).val();
            require (['oxguard/pgp/reader'], function (reader) {
                reader.getInline(baton, password, index);
            });
        };
        baton.data.PGP = true;
        checkSession().then(function () {
            require (['oxguard/mail/oxguard_mail'], function (oxmail) {
                if (window.oxguarddata.passcode === null) {

                    $(location).replaceWith(oxmail.passwordPrompt(baton, false, this, goFunction));
                    baton.view.$el.find('.attachments').hide();
                    if (!_.device('ios')) {
                        baton.view.$el.find('#oxgrpass').focus();
                    }
                    baton.view.$el.show();

                } else {
                    busy (location, oxmail.getId(baton));
                    var go = function (baton, password, index, auth) {
                        require (['oxguard/pgp/reader'], function (reader) {
                            reader.getInline(baton, password, index, auth);
                        });
                    };
                    checkauth (go, baton, index);
                }
            });
        });
    }

    // Mime found, do send for decode
    function pgpFound (baton, location) {
        var goFunction = function (id) {
            $('#error' + id).show();
            var password = ($('#oxgrpass' + id).val() === undefined) ? '' : $('#oxgrpass' + id).val();
            require (['oxguard/pgp/reader'], function (reader) {
                reader.getPGP(baton, password);
            });
        };

        checkSession().then(function () {
            require (['oxguard/mail/oxguard_mail'], function (oxmail) {
                if (window.oxguarddata.passcode === null) {
                    $(location).replaceWith(oxmail.passwordPrompt(baton, false, this, goFunction));
                    baton.view.$el.find('.attachments').remove();
                    baton.view.attachmentView = undefined;
                    if (!_.device('ios')) {
                        baton.view.$el.find('#oxgrpass').focus();
                    }
                    baton.view.$el.show();
                } else {
                    busy (location, oxmail.getId(baton));
                    var go = function (baton, password, index, auth) {
                        require (['oxguard/pgp/reader'], function (reader) {
                            reader.getPGP(baton, password, index, auth);
                        });
                    };
                    checkauth (go, baton, undefined);

                }
            });
        });
    }

    function checkSession () {
        var def = $.Deferred();
        if (!firstload) {
            def.resolve();
            return def;
        }
        require (['oxguard/oxguard_core'], function (core) {
            core.checkAuth()
            .always(function () {
                firstload = false;
                def.resolve();
            });
        });
        return def;
    }

    function checkauth (go, baton, index) {
        if (window.oxguarddata.passcode.indexOf('Password Needed') > -1) {// If no password for user has been defined, open dialog
            require(['oxguard/core/tempPassword'], function (core) {
                core.createOxGuardPasswordPrompt(baton, go, '', undefined, index, location);
            });

            return;
        }
        //FIXME: use util.isGuardConfigured, instead? not sure in this case
        if (window.oxguarddata.passcode.indexOf('No Key') > -1) {// If received a first encrypted email while already logged in, will still have 'no key' in passcode
            require(['oxguard/core/tempPassword'], function (core) {
                core.createOxGuardPasswordPrompt(baton, go, '', undefined, index, location);
            });
            return;
        }
        go (baton, '', index, window.oxguarddata.passcode);
    }

    function pgpAttach(baton) {
        baton.data.PGP = true;
    }

    return {
        checkPGP: checkPGP
    };
});
